@ECHO OFF
:: ====================================================
:: Configuration

:: Path to ImageMagick
set d_ImgMagick=C:\Program Files\ImageMagick

:: Move files to executed directory instead of a "MipMaps" folder.
set d_MoveFiles=false

:: End Configuration
:: ====================================================

:: Some bug? This batch by default tries to run from System32 folder. So a cheap fix, set the
:: batch file's location to a variable, and force the script to start scanning there.
set d_CurDir=%~dp0
CD "%d_CurDir%"

:: Create a folder for the MipMaps so we don't enter an infinite loop.
if not exist "%d_CurDir%\MipMaps" (md "%d_CurDir%\MipMaps")

:: Search for images, use ImageMagick to get width/height, and run image through the external loop.
SetLocal EnableDelayedExpansion
for %%F in (*.png) DO (
    set s_fulName=%%F
	set s_curName=%%~nF
	cd "%d_ImgMagick%"
	for /f "tokens=1-2" %%G in ('identify -ping -format "%%w %%h" "%d_CurDir%\!s_fulName!"') do (
		set i_width=%%G
		set i_height=%%H
	)
	call:GenerateMips !s_curName! !i_width! !i_height!
    cd "%d_CurDir%"
)
goto:EndScript
EndLocal

:: --------------------------------------------------------------------------------------------------
:: This entire chunk is an external loop that is ran within the above loop and generates the mipmaps.

:GenerateMips
:: Set the pre-loop values.
set i_mipMap=0
set s_realName=%~1
set i_DimensionX=%~2
set i_DimensionY=%~3
echo ----------------------------------------------
echo Batching %s_realName%.png
echo.

:StartThis
:: Loop until [8] is the minimum dimension on either axis.
:: For each iteration, the dimensions are halved and i_mipMap is incremented.
:: s_FullDimensions stores X and Y values as 'height'x'width' (example: 512x512)
set /a i_mipMap=i_mipMap+1
set /a i_DimensionX=i_DimensionX/2
set /a i_DimensionY=i_DimensionY/2
set s_FullDimensions=%i_DimensionX%x%i_DimensionY%

:: Create the new mipmap.
"%d_ImgMagick%\convert.exe" "%d_CurDir%\%s_realName%.png" -resize %s_FullDimensions% "%d_CurDir%\MipMaps\%s_realName%_mip%i_mipMap%.png"
echo Creating %s_realName%_mip%i_mipMap%.png ...

:: Repeat the loop if a dimension has not yet reached 8.
if not [%i_DimensionX%] GEQ [8] (
	if not [%i_DimensionY%] GEQ [8] (
		goto:StartThis
	)
)
echo.
echo Total of %i_mipMap% MipMaps created.
goto:eof

:: End of the looping function.
:: --------------------------------------------------------------------------------------------------

:EndScript
:: Move all generated mipmaps to the base directory if desired.
if [%d_MoveFiles%]==[true] (
	cd MipMaps
	for %%F in (*.png) do (move %%F "%d_CurDir%" >nul)
	cd ..
)
echo ----------------------------------------------
echo.
echo Press any key to close this window.
pause >nul
:: Wait until the end to remove the folder to avoid it being in use.
if [%d_MoveFiles%]==[true] (rmdir /S /Q MipMaps) 